package com.carlo.aiwolf.lib.info;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import org.aiwolf.client.lib.*;
import org.aiwolf.common.data.*;

/** 
 *  能力結果発言を管理するクラス
 * @author carlo
 *
 */

public class AbilityInfo {
	/** 各エージェントが発言した能力結果のリストのマップ */
	protected HashMap<Agent,AbilityResultManager> abilityResultMgrMap=new HashMap<Agent,AbilityResultManager>();
	protected SeerDivineTable seerDivineTable;
	protected GameInfoManager gameInfoMgr;
	/** 非村人陣営のリスト。0日目に占い結果を言うなど、破綻発言をしたら追加される。 */
	private List<Agent> nonVillaerTeamAgents;
	
	public AbilityInfo(GameInfoManager gameInfoMgr){
		for(Agent agent:gameInfoMgr.getAgentList()){
			abilityResultMgrMap.put(agent, new AbilityResultManager());
		}
		this.gameInfoMgr=gameInfoMgr;
		this.seerDivineTable=new SeerDivineTable();
		nonVillaerTeamAgents=new ArrayList<Agent>();
	}
	
	public void addAbilityResult(Agent speaker,Utterance utterance,int day){
		//破綻チェック
		if(nonVillaerTeamAgents.contains(speaker)==false){
			//0日目の占い,0・１日目の霊能結果発言は偽
			if((utterance.getTopic()==Topic.DIVINED && day==0) || (utterance.getTopic()==Topic.INQUESTED && (day==0 || day==1))){
				nonVillaerTeamAgents.add(speaker);
			}
			//結果＝人狼発言がゲームの人狼の数を超えていたら破綻
			//これ人狼判定を出した場合にしか計算していないので、CO数が変わった時にも計算が必須。
			if((utterance.getTopic()==Topic.DIVINED || utterance.getTopic()==Topic.INQUESTED) && utterance.getResult()==Species.WEREWOLF){

				//if((abilityResultMgrMap.get(speaker).searchAbilityResultNoOverlap(Species.WEREWOLF).size()+gameInfoMgr.getNumOfWerewolfInCO())>=gameInfoMgr.getRoleNum(Role.WEREWOLF)){
				if((abilityResultMgrMap.get(speaker).searchAbilityResultNoCO(Species.WEREWOLF,gameInfoMgr.getCOInfo()).size()+gameInfoMgr.getNumOfWerewolfInCO())>=gameInfoMgr.getRoleNum(Role.WEREWOLF)){	
					nonVillaerTeamAgents.add(speaker);
				}
			}
		}
			
		if(utterance.getTopic()==Topic.DIVINED) seerDivineTable.addDivineResult(speaker, utterance.getTarget(), utterance.getResult());
		abilityResultMgrMap.get(speaker).addAbilityResult(utterance.getTopic(), -1,day, speaker,utterance.getTarget(),utterance.getResult());
	}
	/** 破綻した（非村人陣営）のエージェントリストを返す */
	public List<Agent> getNonVillagerTeamAgents(){
		return nonVillaerTeamAgents;
	}
	
	/** そのagentの能力行使結果をまとめたクラスを返す */
	public AbilityResultManager getAbilityResultManager(Agent agent){
		return abilityResultMgrMap.get(agent);
	}
	
	public HashMap<Agent,AbilityResultManager> getAbilityResultManagerMap(){
		return abilityResultMgrMap;
	}
	
	/** targetの被占い結果まとめを返す */
	public DivinedResult getDivinedResult(Agent target){
		return seerDivineTable.getDivinedResult(target);
	}
	
	/**
	 * targetを占った占い結果をAbilityResultのリストで返す
	 *  逐次探索しているので、たまに1msくらいかかる
	 * @param target
	 * @return
	 */
	public List<AbilityResult> searchDivinedAbilityResults(Agent target){
		ArrayList<AbilityResult> abilityResults=new ArrayList<AbilityResult>();

		//Targetが一致したAbilityResultを集めて返す
		for(Entry<Agent, AbilityResultManager> entry : abilityResultMgrMap.entrySet()) {
			if(gameInfoMgr.getCOInfo().getCoRole(entry.getKey())!=Role.SEER) continue;
			AbilityResult abilityResult=entry.getValue().getAbilityResult(target);
			if(abilityResult!=null) abilityResults.add(abilityResult);
		}
		return abilityResults;
	}
	/**
	 * 自分以外で、生存中かつCO状態をしていない、占われ状態がtypeのエージェントのリストを返す。
	 * @param type NONE:グレーを返す BLACK:人狼とのみ占われた人物を返す WHITE:人間とのみ占われた人物を返す。 PANDA:２人以上の占い師から人狼、人間と占われた人物を返す
	 * @return
	 */
	public List<Agent> searchDivinedAgents(DivinedType type){
		ArrayList<Agent> agentList=new ArrayList<Agent>();
		for(Agent agent:gameInfoMgr.getAliveAgentList()){
			if(agent==gameInfoMgr.getMyAgent()) continue;
			if(gameInfoMgr.getCOInfo().getCoRole(agent)==null && getDivinedResult(agent).getDivinedType()==type){
				agentList.add(agent);
			}
		}
		return agentList;
	}
	/**
	 * seerが占ってresultだったエージェントのリストを返す
	 * @param seer
	 * @return
	 */
	public List<Agent> searchSeerDivinedAgents(Agent seer,Species result){
		List<Agent> agents=new ArrayList<Agent>();
		for(AbilityResult abilityResult:abilityResultMgrMap.get(seer).getAbilityResultList()){
			if(abilityResult.getSpecies()==result) agents.add(abilityResult.getTarget());
		}
		return agents;
	}
	
	

	
	public void printAbilityResultList(){
		for(Entry<Agent, AbilityResultManager> entry : abilityResultMgrMap.entrySet()) {
			System.out.print(entry.getKey());
			entry.getValue().printList();
			System.out.println();
		}
		
	}

}
