package com.carlo.aiwolf.log;

import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map.Entry;

import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.net.GameInfo;
import org.aiwolf.common.net.GameInfoToSend;

import com.carlo.aiwolf.bayes.trust.TrustLevel;
import com.carlo.aiwolf.bayes.trust.TrustListManager;
import com.carlo.aiwolf.lib.info.GameInfoManager;

import jp.halfmoon.inaba.aiwolf.log.DayLog;
import jp.halfmoon.inaba.aiwolf.log.GameLog;
import jp.halfmoon.inaba.aiwolf.log.LogReader;
import jp.halfmoon.inaba.aiwolf.log.StatusLog;

/**
 * ゲームログから信用度計算を行う
 * @author carlo
 *
 */
public class BayesTester {
	protected TrustListMngForLog trustListManager;
	AgentInfoManagerForLog agentInfo;
	Agent truthSeer;
	Agent truthMedium;
	
	HashMap<Integer,Agent> truthSeerMap=new  HashMap<Integer,Agent>();
	public BayesTester(String filePath) {
		GameLog log = LogReader.getLogData(filePath);
		if( log == null ){
			System.out.println("error!");
			return;
		}
		ArrayList<Agent> agentList=new ArrayList<Agent>();
		for(int i=1;i<16;i++){
			Agent agent = Agent.getAgent(i);
			agentList.add(agent);
			//System.out.println(agent);
			//agentMap.put(i, agent);
		}
		//村人を探して、myAgentとする
		int i=1;
		Agent myAgent=null;
		while(true){
			StatusLog statusLog= log.getDayLog(0).getStatus().get(i++);
			if(statusLog.getRole()==Role.VILLAGER){
				myAgent=Agent.getAgent(statusLog.getAgentNo());
				break;
			}
		}
		agentInfo=new AgentInfoManagerForLog(agentList);
		trustListManager=new TrustListMngForLog(agentList, myAgent, agentInfo);
		
		
		readGameLog(log);
		truthSeer=trustListManager.getRoleCOAgent(TrustLevel.HIGHEST, Role.SEER, false);
		truthMedium=trustListManager.getRoleCOAgent(TrustLevel.HIGHEST, Role.MEDIUM, false);
		
		//trustListManager.printTrustListForCreatingData();
		//System.out.println("真占い師予想:"+trustListManager.getRoleCOAgent(TrustLevel.HIGHEST, Role.SEER, false));
		
	}	
	public GameInfoManager getAgentInfo(){
		return this.agentInfo;
	}
	public Agent getGuessTruthSeer(int day){
		while(true){
			if(truthSeerMap.containsKey(day)) return truthSeerMap.get(day);
			else day--;
		}
	}
	//ゲームログからゲームを再現
	public	void readGameLog(GameLog log){
		truthSeerMap=new  HashMap<Integer,Agent>();
		for (Entry<Integer, DayLog> entry : log.getDays().entrySet()) {
			DayLog dayLog=entry.getValue();
			dayStart(dayLog);
			update(dayLog);
			
			truthSeerMap.put(dayLog.getDay(),trustListManager.getRoleCOAgent(TrustLevel.HIGHEST, Role.SEER, false));
			
		}
		System.out.println(agentInfo.getVoteInfo().searchVoter(3, Agent.getAgent(1)));
	}
	private void update(DayLog dayLog){
		long start = System.currentTimeMillis();
		
		agentInfo.update(dayLog.getTalk());
		trustListManager.update();
		
		long stop = System.currentTimeMillis();
		//System.out.println("update,"+dayLog.getDay()+","+(stop-start)+"ms");
	}
	private void dayStart(DayLog dayLog){
		long start = System.currentTimeMillis();
		
		agentInfo.dayStart(dayLog);
		trustListManager.dayStart(dayLog);
		trustListManager.printTrustList();
		
		long stop = System.currentTimeMillis();
		//System.out.println("dayStart,"+dayLog.getDay()+","+(stop-start)+"ms");
		
	}
	
	
	public static void main(String[] args) throws FileNotFoundException {
		
		
		/*
		for(int day=8;day<9;day++){
			int total=0;
			int correctNum=0;
			for(int i=1;i<=10;i++){
				String fileName="aiwolf"+i;

				//String filePath="./logs/log1209/log/"+fileName+".log";
				String filePath="./logs/comp_data/log"+2+"/aiwolf"+i +".log";
				BayesTester tester=new BayesTester(filePath);
				QuesAnswerCreater answer=new QuesAnswerCreater();
				answer.readGameLog(filePath);
				Agent answerSeer= answer.getTruthSeer();

				//Agent predictSeer=tester.truthSeer;
				Agent predictSeer=tester.getGuessTruthSeer(day);
				//占い師3COのみカウント
				//if(tester.getAgentInfo().countCoAgent(Role.SEER)==3){
				if(answerSeer==predictSeer) correctNum++;
				//else System.out.println("miss:"+fileName);
				total++;
				//}
			}
			System.out.println("day:"+day+" "+correctNum+"/"+total+" :"+(double)correctNum/total);
		}
		*/
		
		/*
		//ファイル出力
		PrintStream out = new PrintStream("result52.arff");
		//置き換える
		System.setOut(out);
		for( int i = 1; i <= 3; i++ ){
			for(int j=1;j<=1000;j++){
		//for(int i=1;i<=200;i++){
			String fileName="aiwolf"+j;

			String filePath="./logs/comp_data/log"+i+"/aiwolf" + j + ".log";
			//String filePath="./logs/log1209/log/"+fileName+".log";
			BayesTester tester=new BayesTester(filePath);
			QuesAnswerCreater answer=new QuesAnswerCreater();
			answer.readGameLog(filePath);

			
			}
		}
		*/
		
		//1回実行
		
		for(int i=1;i<2;i++){
			String fileName="aiwolf"+i;


			String filePath="./logs/log1209/log/"+fileName+".log";
			System.out.println(filePath);
			BayesTester tester=new BayesTester(filePath);
			QuesAnswerCreater answer=new QuesAnswerCreater();
			answer.readGameLog(filePath);
		}
		//Agent answerSeer= answer.getTruthSeer();
		//Agent predictSeer=tester.truthSeer;
		//Agent answerMedium=answer.getTruthMedium();
		//Agent predictMedium=tester.truthMedium;
		
		//System.out.println("medium ans:"+answerMedium+" pred:"+predictMedium+""+(answerMedium==predictMedium));
		//System.out.println("seer ans:"+answerSeer+" pred:"+predictSeer+""+(answerSeer==predictSeer));
		
	}

}
