package com.carlo.aiwolf.log;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.aiwolf.client.lib.TemplateTalkFactory.TalkType;
import org.aiwolf.client.lib.Topic;
import org.aiwolf.client.lib.Utterance;
import org.aiwolf.common.data.*;
import org.aiwolf.common.net.GameInfo;

import com.carlo.aiwolf.bayes.trust.*;
import com.carlo.aiwolf.bayes.trust.noweka.NoWekaTrustList;
import com.carlo.aiwolf.lib.info.AbilityResult;
import com.carlo.aiwolf.lib.info.GameInfoManager;

import jp.halfmoon.inaba.aiwolf.log.DayLog;
import jp.halfmoon.inaba.aiwolf.log.TalkLog;

/**
 * 　ログで動作するように、TrustListManagerのメソッドをオーバーライドしたもの
 * @author carlo
 *
 */
public class TrustListMngForLog extends TrustListManager{
	private DayLog dayLog;
	public TrustListMngForLog(List<Agent> agentList, Agent myAgent, GameInfoManager agentInfo) {
		super(agentList, myAgent, agentInfo);
		setShowConsoleLog(false);
		//trustList=new TrustList(agentList,myAgent,agentInfo);
		trustList=new NoWekaTrustList(agentList,myAgent,agentInfo);
		trustList.setShowConsoleLog(false);
	}
	public void dayStart(DayLog dayLog){
		this.dayLog=dayLog;
		super.dayStart();
		gameInfoMgr.getCOInfo().printCoRoleMap();
		//trustList.getTrustPoint(Agent.getAgent(1));
	}
	@Override
	protected int getDay(){
		return dayLog.getDay();
	}
	/** ログからRoleMapを生成して、動作させる<br>
	 * isShowConsoleLogを無視して表示 */
	public void printTrustListForCreatingData(){
		Map<Agent,Role> roleMap=new HashMap<Agent,Role>();
		//map生成
		for(int i=1;i<=15;i++){
			roleMap.put(Agent.getAgent(i), dayLog.getStatus().get(i).getRole());
		}
		trustList.printTrustListForCreatingData(roleMap);
	}
	@Override
	protected void readTalkList(){
		dayLog.getTalk();
		//talkLog-List変換
		ArrayList<Talk> talkList=new ArrayList<Talk>();
		for(TalkLog talkLog:dayLog.getTalk()){
			if(talkLog.getTalkType()==TalkType.TALK){
				Talk talk=new Talk(talkLog.getIdx(),talkLog.getDay(),Agent.getAgent(talkLog.getAgentNo()),talkLog.getContent());
				talkList.add(talk);
			}
		}
		long before=0;
		Topic beforeTopic=null;
		
		//後は一緒
		for(int i=readTalkNum;i<talkList.size();i++){
			Talk talk=talkList.get(i);
			Utterance utterance=new Utterance(talk.getContent());
			
			//時間計測
			//if(before!=0 && utterance.getTopic()!=Topic.OVER && utterance.getTopic()!=Topic.SKIP){
			long now = System.currentTimeMillis();
			//System.out.println(utterance.get(Topic());
			if(beforeTopic!=Topic.OVER && beforeTopic!=Topic.SKIP){
				//System.out.println((i-1)+""+beforeTopic+(now-before)+"ms");
			}
			before=now;
			beforeTopic=utterance.getTopic();
			//}
			//else before=System.currentTimeMillis();
			
			switch (utterance.getTopic()){
			case COMINGOUT:
				switch (utterance.getRole()){
				case MEDIUM:
					break;
				default:
					break;
				}
				break;
			case DIVINED:
				
				Agent seer=talk.getAgent();
				int day=talk.getDay();
				Species species=utterance.getResult();
				trustList.changeSeerTrust(seer,day, species, Correct.UNKNOWN,utterance.getTarget(),Correct.UNKNOWN);
				
				break;
			case INQUESTED:
				Agent medium=talk.getAgent();
				//霊能者のネットワーク計算
				trustList.changeMediumTrust(medium, utterance.getResult(), talk.getDay());
				
				//霊能COが一人だけなら、霊能を真と仮定して各占いとラインが繋がっているかで信用度の計算
				if(gameInfoMgr.getCOInfo().getNumOfCOAgent(Role.MEDIUM)==1){
					Species inquestedResult=utterance.getResult();
					//投票結果から計算
					for(Entry<Agent,Integer> voteEntry : gameInfoMgr.getVoteInfo().searchVoterMap(utterance.getTarget()).entrySet()) {
						int voteDay=voteEntry.getValue();
						Agent voteAgent=voteEntry.getKey();
						Agent executedAgent=utterance.getTarget();
						boolean assist= (executedAgent==gameInfoMgr.getVoteInfo().getVoteTarget(voteDay, voteAgent));
						trustList.changeVoterTrust(voteEntry.getKey(), utterance.getResult(),voteDay,String.valueOf(assist));
					}
					
					//霊媒先と一致する占い結果を探す
					for(AbilityResult abilityResult:gameInfoMgr.getAbilityInfo().searchDivinedAbilityResults(utterance.getTarget())){
						//まず前回の結果を戻す
						
						//if(isShowConsoleLog) System.out.println("back trust ");
						trustList.changeSeerTrust(abilityResult.getAgent(),abilityResult.getTalkedDay(), abilityResult.getSpecies(), Correct.UNKNOWN,true,utterance.getTarget(),Correct.UNKNOWN);
						
						//判明した結果を入れて計算
						Correct correct=Correct.UNKNOWN;
						if(inquestedResult==abilityResult.getSpecies()) correct=Correct.YES;
						else correct=Correct.NO;
						
						//if(isShowConsoleLog) System.out.println("recalc trust ");
						trustList.changeSeerTrust(abilityResult.getAgent(),abilityResult.getTalkedDay(), abilityResult.getSpecies(), correct,utterance.getTarget(),Correct.UNKNOWN);
						
					}
				}
				break;
			default:
				break;

			}
			readTalkNum++;
		}
	}

}
