package com.koizu.aiwolf.player;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.aiwolf.client.base.player.AbstractPossessed;
import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Judge;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.data.Species;
import org.aiwolf.common.net.GameInfo;
import org.aiwolf.common.net.GameSetting;

import com.carlo.aiwolf.lib.ExRandom;
import com.carlo.aiwolf.lib.info.CauseOfDeath;
import com.carlo.aiwolf.lib.info.GameInfoManager;
import com.carlo.aiwolf.lib.talk.FakeCOTalkCreator;
import com.carlo.aiwolf.lib.talk.TalkCreator;

/**
 * 狂人
 * 
 * @author koizu
 *
 */
public class Possessed extends AbstractPossessed {
	protected GameInfoManager gameInfoMgr;
	private FakeCOTalkCreator talkCreater;
	private  ArrayList<Judge> fakeJudgeList;
	private List<Agent> fakeTargetList;//偽能力行使対象
	private int fakeWolfNumber = 0;//嘘の占い結果で人狼と言った回数
	private Agent heIsWolf;//その日に占い結果で人狼がでたと発言した対象。日が進む時にnullに戻す必要がある
	//private List<Agent> heIshumanList;//占い結果で人間だったと発言した対象のリスト*未使用
	private List<Agent> voteList;//投票対象のリスト
	Random rnd;//混合戦略用
	
	@Override
	public void initialize(GameInfo gameInfo,GameSetting gameSetting){
		super.initialize(gameInfo, gameSetting);
		rnd = new Random();
		gameInfoMgr=new GameInfoManager(gameInfo,this.getMe());
		fakeJudgeList=new ArrayList<Judge>();
		talkCreater=new FakeCOTalkCreator(this);
		
		//占い師を騙る
		talkCreater.setFakeRole(Role.SEER);
		
		fakeTargetList = gameInfoMgr.getAgentListExceptMe(true);
		voteList       = gameInfoMgr.getAgentListExceptMe(true);
	}


	@Override
	public void update(GameInfo gameInfo){
		super.update(gameInfo);
		gameInfoMgr.update(gameInfo);
		talkCreater.setVoteTarget(thinkVote());
		
		talkCreater.doComingOut();
	}
	
	
	
	@Override
	public void finish() {
		// TODO Auto-generated method stub

	}

	@Override
	public String talk() {
		return talkCreater.talk();
		// TODO Auto-generated method stub
		//return null;
	}

	@Override
	public Agent vote() {
		return talkCreater.getVoteTarget();
		// TODO Auto-generated method stub
		//return null;
	}

	@Override
	public void dayStart() {
		
		gameInfoMgr.dayStart();
		
		
		heIsWolf = null;
		
		//投票対象から前日に死んだ人を排除
		if(getDay()>0){
			voteList.remove(gameInfoMgr.getDeathInfo().getDeadAgent(gameInfoMgr.getDay()-1,CauseOfDeath.EXECUTED));
			voteList.remove(gameInfoMgr.getDeathInfo().getDeadAgent(gameInfoMgr.getDay()-1,CauseOfDeath.ATTACKED));
		}
		
		
		//偽能力を行使していない対象から対象を選ぶ
		//偽の結果作成
		if(getDay()>0){
			//偽能力行使対象リストから前日に死んだ人を排除
			fakeTargetList.remove(gameInfoMgr.getDeathInfo().getDeadAgent(gameInfoMgr.getDay()-1,CauseOfDeath.EXECUTED));
			fakeTargetList.remove(gameInfoMgr.getDeathInfo().getDeadAgent(gameInfoMgr.getDay()-1,CauseOfDeath.ATTACKED));
			Agent target;
			//ターゲットをリストからランダムに選択
			if(fakeTargetList.size() > 0){
				target = ExRandom.select(fakeTargetList);
			}else{
				target = gameInfoMgr.getMyAgent();
			}
				
			int ran = rnd.nextInt(10);
			if(getDay()>=4){//終盤は人狼率が高くなるので占い結果は人間と発言する
				Judge judge=new Judge(getDay(),getMe(),target,Species.HUMAN);
				fakeJudgeList.add(judge);
				//voteList.remove(target);//一度占い結果で人間と言った対象には投票しない→終盤は気にしない
			}
			else if(ran >= 7 && fakeWolfNumber < 3){//３割の確率で人狼。ただし結果を人狼とするのは2回まで。できればゲーム中の人狼の数に合わせるように改良したい。
				Judge judge=new Judge(getDay(),getMe(),target,Species.WEREWOLF);
				fakeJudgeList.add(judge);
				fakeWolfNumber++;
				heIsWolf = target;
			}
			else{
				Judge judge=new Judge(getDay(),getMe(),target,Species.HUMAN);
				fakeJudgeList.add(judge);
				//heIshumanList.add(target);*未使用
				voteList.remove(target);//一度占い結果で人間と言った対象には投票しない
			}		
			
			fakeTargetList.remove(target);//偽能力行使対象リストから今回の対象を削除
			
		
		}
		talkCreater.dayStart(fakeJudgeList);
		
		// TODO Auto-generated method stub

	}
	
	private Agent thinkVote(){
		
		
		if(heIsWolf != null){//その日占い結果で人狼が出たと発言した時、その対象に投票
			return heIsWolf;
		}
		else if(voteList.size()>0){//自分が人間だと主張した人物以外に投票
			return ExRandom.select(voteList);
			//return voteList.get(0);
		}
		else {//例外
			//return gameInfoMgr.getAgentListExceptMe(true).get(0);
			return ExRandom.select(gameInfoMgr.getAgentListExceptMe(true));
		}
		
		
	}

}
