package com.carlo.aiwolf.bayes.player;

import java.util.ArrayList;
import java.util.List;

import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.data.Species;
import org.aiwolf.common.net.GameInfo;
import org.aiwolf.common.net.GameSetting;

import com.carlo.aiwolf.bayes.trust.TrustLevel;
import com.carlo.aiwolf.bayes.trust.TrustListManager;
import com.carlo.aiwolf.lib.ExRandom;
import com.carlo.aiwolf.lib.info.*;
import com.carlo.aiwolf.lib.talk.TalkCreator;

/**
 * 各roleクラスで使う共通部分をくくりだしたクラス
 * @author carlo
 *
 */

public class GameBrain {
	private GameInfoManager gameInfoMgr;
	private TrustListManager trustListMgr;
	private TalkCreator talkCreater;
	/** 新占い */
	private Agent truthSeer;
	private Agent truthMedium;
	public GameBrain(GameInfoManager gameInfoMgr,TrustListManager trustListMgr,TalkCreator talkCreater){
		truthSeer=null;
		truthMedium=null;
		this.gameInfoMgr=gameInfoMgr;
		this.trustListMgr=trustListMgr;
		this.talkCreater=talkCreater;
	}
	/** 投票先を考えて返す */
	public Agent thinkVote(){
		searchTruthSeer();
		searchTruthMedium();
		
		List<Agent> voteTargets;
		List<Agent> dontVoteTargets=new ArrayList<Agent>();
		//真占いがいるなら
		if(truthSeer!=null){
			//真占いの黒は投票
			voteTargets=gameInfoMgr.getAbilityInfo().searchSeerDivinedAgents(truthSeer, Species.WEREWOLF);
			if(voteTargets.size()>0) return ExRandom.select(voteTargets, talkCreater.getVoteTarget());
			//白には投票しない
			dontVoteTargets.add(truthSeer);
			dontVoteTargets.addAll(gameInfoMgr.getAbilityInfo().searchSeerDivinedAgents(truthSeer, Species.HUMAN));
		}
		//真霊能がいるなら
		if(truthMedium!=null){
			dontVoteTargets.add(truthMedium);
		}
		
		//破綻者に投票
		voteTargets=gameInfoMgr.getAbilityInfo().getNonVillagerTeamAgents();
		if(voteTargets.size()>0) return ExRandom.select(voteTargets, talkCreater.getVoteTarget());
		
		//信用度30以下は投票
		voteTargets=trustListMgr.getSortedAgentList(true, 30);
		for(Agent target:voteTargets){
			//投票してはいけない相手でなければ
			if(dontVoteTargets.contains(target)==false) return target;
		}
		/** 3日目までに暫定黒がいれば投票 */
		if(gameInfoMgr.getDay()>0 && gameInfoMgr.getDay()<4){
			for(Agent agent:gameInfoMgr.getCOInfo().getCOAgentList(null, true, true)){
				if(gameInfoMgr.getAbilityInfo().getDivinedResult(agent).getDivinedType()==DivinedType.BLACK){
					return agent;
				}
			}
		}
		
		
		/** 1,2日目は純粋にグレラン。いなければ次へ */
		if(gameInfoMgr.getDay()<3){
			voteTargets=gameInfoMgr.getAbilityInfo().searchDivinedAgents(DivinedType.NONE);
			for(Agent target:voteTargets){
				//投票してはいけない相手でなければ
				if(dontVoteTargets.contains(target)==false) return target;
			}
		}
		/** 3,4日目は非CO者から信用度が低いものを。いなければ次へ */
		if(gameInfoMgr.getDay()<5) {
			voteTargets=trustListMgr.getSortedRoleCOAgentList(null, true);
			for(Agent target:voteTargets){
				//投票してはいけない相手でなければ
				if(dontVoteTargets.contains(target)==false) return target;
			}
		}
		/** それ以降は信用度が低いものを */
		voteTargets= trustListMgr.getSortedAgentList(true);
		for(Agent target:voteTargets){
			//投票してはいけない相手でなければ
			if(dontVoteTargets.contains(target)==false) return target;
		}
		return trustListMgr.getAgent(TrustLevel.LOWEST, true);
	}
	public Agent getTruthSeer(){
		return truthSeer;
	}
	public Agent getTruthMedium(){
		return truthMedium;
	}
	/** 正答率確かめ用 */
	public void finish(GameInfo gameInfo){
		//if(truthSeer!=null) System.out.println(gameInfo.getRole()+",SEER,"+gameInfo.getRoleMap().get(truthSeer));
		//if(truthMedium!=null) System.out.println(gameInfo.getRole()+",MEDIUM,"+gameInfo.getRoleMap().get(truthMedium));
	}
	
	/** 真占いを探す */
	private void searchTruthSeer(){
		if(truthSeer==null){
			if(gameInfoMgr.getMyRole()==Role.SEER) {
				truthSeer=gameInfoMgr.getMyAgent();
				return;
			}
			Agent mostTrustedSeer=trustListMgr.getRoleCOAgent(TrustLevel.HIGHEST, Role.SEER,false);
			if(mostTrustedSeer!=null &&  trustListMgr.getTrustPoint(mostTrustedSeer)>85){
				truthSeer=mostTrustedSeer;
				talkCreater.setEstimate(truthSeer, Role.SEER);
				return;
			}
		}
	}
	/** 真霊能を探す */
	private void searchTruthMedium(){
		if(truthMedium==null){
			//自分が霊能なら、真霊能は自分
			if(gameInfoMgr.getMyRole()==Role.MEDIUM) {
				truthMedium=gameInfoMgr.getMyAgent();
				return;
			}
			
			Agent mostTrustedMedium=trustListMgr.getRoleCOAgent(TrustLevel.HIGHEST, Role.MEDIUM, false);
			//信用度が70超えてれば真認定
			if(mostTrustedMedium!=null && trustListMgr.getTrustPoint(mostTrustedMedium)>75){
				truthMedium=mostTrustedMedium;
				trustListMgr.setTruthMedium(truthMedium);
				//System.out.print(truthMedium);
				talkCreater.setEstimate(truthMedium, Role.MEDIUM);
				return;
			}
			//4日目以降で一人COかつ信用度50超えなら真認定
			if(mostTrustedMedium!=null && gameInfoMgr.getDay()>3 && gameInfoMgr.getCOInfo().getNumOfCOAgent(Role.MEDIUM)==1 &&
					trustListMgr.getTrustPoint(mostTrustedMedium)>50){
				truthMedium=mostTrustedMedium;
				trustListMgr.setTruthMedium(truthMedium);
				//System.out.print(truthMedium);
				talkCreater.setEstimate(truthMedium, Role.MEDIUM);
				return;
			}
		}
	}

}
