package com.carlo.aiwolf.sample.player;

import java.util.List;

import org.aiwolf.client.base.player.AbstractBodyguard;
import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.net.GameInfo;
import org.aiwolf.common.net.GameSetting;

import com.carlo.aiwolf.lib.ExRandom;
import com.carlo.aiwolf.lib.info.GameInfoManager;
import com.carlo.aiwolf.lib.info.TalkInfo;
import com.carlo.aiwolf.lib.talk.COTalkCreator;


public class SampleBodyguard extends AbstractBodyguard {
	private GameInfoManager gameInfoMgr;
	
	private COTalkCreator talkCreator;

	@Override
	public void initialize(GameInfo gameInfo,GameSetting gameSetting){
		super.initialize(gameInfo, gameSetting);
		gameInfoMgr=new GameInfoManager(gameInfo,this.getMe());
		talkCreator=new COTalkCreator(this);
	}
	@Override
	public void update(GameInfo gameInfo){
		super.update(gameInfo);
		gameInfoMgr.update(gameInfo);
		talkCreator.setVoteTarget(thinkVote());
		//CO条件設定
		TalkInfo talkManager=gameInfoMgr.getTalkInfo();
		//1日目以降かつ半数以上が投票先発言をしていて、投票数トップならCO
		if(getDay()>0 && talkManager.getCountTodayVotes()>gameInfo.getAliveAgentList().size()/2 && talkManager.searchAgentListMostVoted().contains(getMe())){
			talkCreator.doComingOut();
		}
		
	}

	@Override
	public void dayStart() {
		gameInfoMgr.dayStart();
		talkCreator.dayStart(this.getLatestDayGameInfo().getGuardedAgent());
		
		
	}

	@Override
	public void finish() {
	}

	@Override
	public String talk() {
		return talkCreator.talk();
	}

	@Override
	public Agent vote() {
		return talkCreator.getVoteTarget();
	}
	@Override
	public Agent guard() {
		List<Agent> agents;
		agents=gameInfoMgr.getCOInfo().getCOAgentList(Role.SEER, true, false);
		if(agents.size()>0) return ExRandom.select(agents);
		
		return ExRandom.select(gameInfoMgr.getAgentListExceptMe(true));
	}
	
	private Agent thinkVote(){
		List<Agent> voteTargets;
		
		//占い師が4人以上でたらローラー
		if(gameInfoMgr.getCOInfo().getNumOfCOAgent(Role.SEER)>=4){
			voteTargets=gameInfoMgr.getCOInfo().getCOAgentList(Role.SEER, true,true);
			if(voteTargets.size()>0) return voteTargets.get(0);
		}
		//霊能者が2人以上でたらローラー
		if(gameInfoMgr.getCOInfo().getNumOfCOAgent(Role.MEDIUM)>=2){
			voteTargets= gameInfoMgr.getCOInfo().getCOAgentList(Role.MEDIUM, true,true);
			if(voteTargets.size()>0) return voteTargets.get(0);
		}
		
		//3日までは非CO者からランダムに選択
		if(getDay()<4){
			voteTargets=gameInfoMgr.getCOInfo().getCOAgentList(null, true,true);
			if(voteTargets.size()>0) return ExRandom.select(voteTargets, talkCreator.getVoteTarget());
		}
		//上記条件に引っかからなければ全員からランダムに選択
		voteTargets=gameInfoMgr.getAgentListExceptMe(true);
		return ExRandom.select(voteTargets, talkCreator.getVoteTarget());
	}



}
